/*
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libxfce4util/libxfce4util.h>
#include "libappstarter/xfce_framebox.h"
#include "libappstarter/as.h"

#define PLUGIN "vlc"

typedef struct _VlcPlugin {
    gchar *path;
    int    fullscreen;
    int    loop;
} VlcPlugin;

VlcPlugin *plugin;
gchar *cmd = NULL;
gchar *conffile = NULL;

static void
read_config ()
{
    XfceRc *rc;

    rc = as_rc_open (conffile, TRUE);
    if (!rc)
        return;

    xfce_rc_set_group (rc, PLUGIN);
    plugin->path = g_strdup (
        xfce_rc_read_entry (rc, "path", ""));
    plugin->fullscreen = xfce_rc_read_int_entry (rc, "fullscreen", 1);
    plugin->loop = xfce_rc_read_int_entry (rc, "loop", 1);
    xfce_rc_close (rc);
}

static void
write_config ()
{
    XfceRc *rc;

    rc = as_rc_open (conffile, FALSE);
    if (!rc)
        return;

    xfce_rc_set_group (rc, PLUGIN);
    xfce_rc_write_entry (rc, "path", plugin->path);
    xfce_rc_write_int_entry (rc, "fullscreen", plugin->fullscreen);
    xfce_rc_write_int_entry (rc, "loop", plugin->loop);

    xfce_rc_close (rc);
}

/* callbacks */
static void
entry_changed (GtkWidget *fe, gpointer data)
{
    g_free (plugin->path);
    plugin->path = g_strdup (as_file_entry_get_filename (
                             AS_FILE_ENTRY (fe)));
    write_config();
}

static void
chk_toggled (GtkWidget *chk, gpointer data)
{
    gchar *d = (gchar *) g_object_get_data (G_OBJECT (chk), "as-id");
    if (strncmp (d, "loop", 4) == 0)
        plugin->loop = (gtk_toggle_button_get_active (
            GTK_TOGGLE_BUTTON (chk))) ? 1 : 0;
    else if (strncmp (d, "fullscreen", 10) == 0)
        plugin->fullscreen = (gtk_toggle_button_get_active (
            GTK_TOGGLE_BUTTON (chk))) ? 1 : 0;
    write_config();
}

GtkWidget *
get_config_widget (gchar *file)
{
    GtkWidget *vbox, *vbox2;
    GtkWidget *frame, *label;
    GtkWidget *fs_chk, *loop_chk;
    GtkWidget *fe;

    conffile = g_strdup (file);
    plugin = g_new0(VlcPlugin, 1);
    plugin->path = NULL;
    read_config ();

    vbox = gtk_vbox_new (FALSE, 2);

    vbox2 = gtk_vbox_new (FALSE, 2);
    gtk_widget_show (vbox2);

    label = gtk_label_new (_("Select a file or a playlist: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (vbox2), label, FALSE, FALSE, 2);
    gtk_widget_show (label);

    fe = as_file_entry_new (_("Select a file"), GTK_FILE_CHOOSER_ACTION_OPEN,
        (gchar *) plugin->path);
    gtk_box_pack_start (GTK_BOX (vbox2), fe, FALSE, FALSE, 2);
    gtk_widget_show (fe);

    frame = xfce_framebox_new (_("File selection"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start(GTK_BOX (vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    vbox2 = gtk_vbox_new (FALSE, 2);
    gtk_widget_show (vbox2);
    fs_chk = gtk_check_button_new_with_label (_("Start in full screen"));
    g_object_set_data (G_OBJECT (fs_chk), "as-id", "fullscreen");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fs_chk),
        (plugin->fullscreen) ? TRUE : FALSE);
    gtk_box_pack_start (GTK_BOX (vbox2), fs_chk, FALSE, FALSE, 2);
    gtk_widget_show (fs_chk);
    loop_chk = gtk_check_button_new_with_label (_("Loop"));
    g_object_set_data (G_OBJECT (loop_chk), "as-id", "loop");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (loop_chk),
        (plugin->loop) ? TRUE : FALSE);
    gtk_box_pack_start (GTK_BOX (vbox2), loop_chk, FALSE, FALSE, 2);
    gtk_widget_show (loop_chk);

    frame = xfce_framebox_new (_("Misc"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start(GTK_BOX (vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    g_signal_connect (G_OBJECT (fe), "changed",
        G_CALLBACK (entry_changed), NULL);

    g_signal_connect (G_OBJECT (fs_chk), "toggled",
        G_CALLBACK (chk_toggled), NULL);
    g_signal_connect (G_OBJECT (loop_chk), "toggled",
        G_CALLBACK (chk_toggled), NULL);


    return vbox;
}

gchar *
get_cmd()
{
    g_free (cmd);
    cmd = g_strconcat ("vlc ",
                       plugin->fullscreen ? "-f " : "",
                       plugin->loop ? "-L " : "",
                       plugin->path,
                       NULL);

    return cmd;
}

void
free_all()
{
    g_free (cmd);
    g_free (plugin->path);
    g_free (plugin);
    g_free (conffile);
}
